@@ -12,8 +12,8 @@
 #include "pythoncapi_compat.h"
 #include "bitarray.h"
 
-/* set using the Python module function _set_bato() */
-static PyObject *bitarray_type_obj = NULL;
+/* set during module initialization */
+static PyObject *bitarray_type_obj;
 
 /* Return 0 if obj is bitarray.  If not, return -1 and set an exception. */
 static int
@@ -21,8 +21,6 @@ ensure_bitarray(PyObject *obj)
 {
     int t;
 
-    if (bitarray_type_obj == NULL)
-        Py_FatalError("bitarray_type_obj not set");
     t = PyObject_IsInstance(obj, bitarray_type_obj);
     if (t < 0)
         return -1;
@@ -34,19 +32,6 @@ ensure_bitarray(PyObject *obj)
     return 0;
 }
 
-/* ensure object is a bitarray of given length */
-static int
-ensure_ba_of_length(PyObject *a, Py_ssize_t n)
-{
-    if (ensure_bitarray(a) < 0)
-        return -1;
-    if (((bitarrayobject *) a)->nbits != n) {
-        PyErr_SetString(PyExc_ValueError, "size mismatch");
-        return -1;
-    }
-    return 0;
-}
-
 /* ------------------------------- count_n ----------------------------- */
 
 /* return the smallest index i for which a.count(vi, 0, i) == n, or when
@@ -108,20 +93,17 @@ count_to_n(bitarrayobject *a, Py_ssize_t n, int vi)
 static PyObject *
 count_n(PyObject *module, PyObject *args)
 {
-    PyObject *value = Py_True, *a;
+    PyObject *a;
     Py_ssize_t n, i;
-    int vi;
+    int vi = 1;
 
-    if (!PyArg_ParseTuple(args, "On|O:count_n", &a, &n, &value))
-        return NULL;
-    if (ensure_bitarray(a) < 0)
+    if (!PyArg_ParseTuple(args, "O!n|O&:count_n",
+                          bitarray_type_obj, &a, &n, conv_pybit, &vi))
         return NULL;
     if (n < 0) {
         PyErr_SetString(PyExc_ValueError, "non-negative integer expected");
         return NULL;
     }
-    if ((vi = pybit_as_int(value)) < 0)
-        return NULL;
 
 #define aa  ((bitarrayobject *) a)
     if (n > aa->nbits)  {
@@ -201,15 +183,12 @@ find_last(bitarrayobject *self, int vi, Py_ssize_t a, Py_ssize_t b)
 static PyObject *
 r_index(PyObject *module, PyObject *args)
 {
-    PyObject *value = Py_True, *a;
+    PyObject *a;
     Py_ssize_t start = 0, stop = PY_SSIZE_T_MAX, res;
-    int vi;
+    int vi = 1;
 
-    if (!PyArg_ParseTuple(args, "O|Onn:rindex", &a, &value, &start, &stop))
-        return NULL;
-    if (ensure_bitarray(a) < 0)
-        return NULL;
-    if ((vi = pybit_as_int(value)) < 0)
+    if (!PyArg_ParseTuple(args, "O!|O&nn:rindex", bitarray_type_obj, &a,
+                          conv_pybit, &vi, &start, &stop))
         return NULL;
 
 #define aa  ((bitarrayobject *) a)
@@ -273,9 +252,8 @@ binary_function(PyObject *args, enum kernel_type kern, const char *format)
     unsigned char c;
     int r;
 
-    if (!PyArg_ParseTuple(args, format, &a, &b))
-        return NULL;
-    if (ensure_bitarray(a) < 0 || ensure_bitarray(b) < 0)
+    if (!PyArg_ParseTuple(args, format,
+                          bitarray_type_obj, &a, bitarray_type_obj, &b))
         return NULL;
 
 #define aa  ((bitarrayobject *) a)
@@ -353,7 +331,7 @@ binary_function(PyObject *args, enum kernel_type kern, const char *format)
 static PyObject *                                                       \
 count_ ## oper (PyObject *module, PyObject *args)                       \
 {                                                                       \
-    return binary_function(args, KERN_c ## oper, "OO:count_" #oper);    \
+    return binary_function(args, KERN_c ## oper, "O!O!:count_" #oper);  \
 }                                                                       \
 PyDoc_STRVAR(count_ ## oper ## _doc,                                    \
 "count_" #oper "(a, b, /) -> int\n\
@@ -369,7 +347,7 @@ COUNT_FUNC(xor, "^");           /* count_xor */
 static PyObject *
 subset(PyObject *module, PyObject *args)
 {
-    return binary_function(args, KERN_subset, "OO:subset");
+    return binary_function(args, KERN_subset, "O!O!:subset");
 }
 
 PyDoc_STRVAR(subset_doc,
@@ -399,7 +377,7 @@ serialize(PyObject *module, PyObject *a)
 
     str = PyBytes_AsString(result);
 #define aa  ((bitarrayobject *) a)
-    *str = (char) (16 * IS_BE(aa) + setunused(aa));
+    *str = (char) (16 * IS_BE(aa) + set_padbits(aa));
     memcpy(str + 1, aa->ob_item, (size_t) nbytes);
 #undef aa
     return result;
@@ -484,12 +462,14 @@ hex2ba(PyObject *module, PyObject *args)
     Py_ssize_t i, strsize;
     int le, be;
 
-    if (!PyArg_ParseTuple(args, "Os#", &a, &str, &strsize))
-        return NULL;
-    if (ensure_ba_of_length(a, 4 * strsize) < 0)
+    if (!PyArg_ParseTuple(args, "O!s#", bitarray_type_obj, &a, &str, &strsize))
         return NULL;
 
 #define aa  ((bitarrayobject *) a)
+    if (aa->nbits != 4 * strsize) {
+        PyErr_SetString(PyExc_ValueError, "size mismatch");
+        return NULL;
+    }
     le = IS_LE(aa);
     be = IS_BE(aa);
     assert(le + be == 1 && str[strsize] == 0);
@@ -584,12 +564,10 @@ ba2base(PyObject *module, PyObject *args)
     char *str;
     int n, m, le;
 
-    if (!PyArg_ParseTuple(args, "iO:ba2base", &n, &a))
+    if (!PyArg_ParseTuple(args, "iO!:ba2base", &n, bitarray_type_obj, &a))
         return NULL;
     if ((m = base_to_length(n)) < 0)
         return NULL;
-    if (ensure_bitarray(a) < 0)
-        return NULL;
 
     switch (n) {
     case 32: alphabet = base32_alphabet; break;
@@ -648,16 +626,19 @@ base2ba(PyObject *module, PyObject *args)
     char *str = NULL;
     int n, m, le;
 
-    if (!PyArg_ParseTuple(args, "i|Os#", &n, &a, &str, &strsize))
+    if (!PyArg_ParseTuple(args, "i|O!s#",
+                          &n, bitarray_type_obj, &a, &str, &strsize))
         return NULL;
     if ((m = base_to_length(n)) < 0)
         return NULL;
     if (a == NULL)
         return PyLong_FromLong(m);
-    if (ensure_ba_of_length(a, m * strsize) < 0)
-        return NULL;
 
 #define aa  ((bitarrayobject *) a)
+    if (aa->nbits != m * strsize) {
+        PyErr_SetString(PyExc_ValueError, "size mismatch");
+        return NULL;
+    }
     le = IS_LE(aa);
     for (i = 0; i < strsize; i++) {
         int j, d = digit_to_int(n, str[i]);
@@ -704,14 +685,16 @@ grow_buffer(bitarrayobject *self)
     return 0;
 }
 
-/* PADBITS is always 3 - the number of bits that represent the number of
-   padding bits.  The actual number of padding bits is called 'padding'
-   below, and is in range(0, 7).
-   Also note that 'padding' refers to the pad bits within the variable
-   length format, which is not the same as the pad bits of the actual
-   bitarray.  For example, b'\x10' has padding = 1, and decodes to
-   bitarray('000'), which has 5 pad bits. */
-#define PADBITS  3
+/* LEN_PAD_BITS is always 3 - the number of bits (length) that is necessary to
+   represent the number of pad bits.  The number of padding bits itself is
+   called 'padding' below.
+
+   'padding' refers to the pad bits within the variable length format.
+   This is not the same as the pad bits of the actual bitarray.
+   For example, b'\x10' has padding = 1, and decodes to bitarray('000'),
+   which has 5 pad bits.  'padding' can take values to up 6.
+ */
+#define LEN_PAD_BITS  3
 
 /* consume iterator while decoding bytes into bitarray */
 static PyObject *
@@ -723,7 +706,7 @@ vl_decode(PyObject *module, PyObject *args)
     unsigned char b = 0x80;  /* empty stream will raise StopIteration */
     Py_ssize_t k;
 
-    if (!PyArg_ParseTuple(args, "OO", &iter, &a))
+    if (!PyArg_ParseTuple(args, "OO!", &iter, bitarray_type_obj, &a))
         return NULL;
     if (!PyIter_Check(iter))
         return PyErr_Format(PyExc_TypeError, "iterator or bytes expected, "
@@ -776,7 +759,7 @@ vl_decode(PyObject *module, PyObject *args)
 
     if (b & 0x80)
         return PyErr_Format(PyExc_StopIteration, "no terminating byte found, "
-                            "bytes read: %zd", (i + PADBITS) / 7);
+                            "bytes read: %zd", (i + LEN_PAD_BITS) / 7);
 
     Py_RETURN_NONE;
 }
@@ -793,9 +776,9 @@ vl_encode(PyObject *module, PyObject *a)
         return NULL;
 
 #define aa  ((bitarrayobject *) a)
-    n = (aa->nbits + PADBITS + 6) / 7;  /* number of resulting bytes */
-    m = 7 * n - PADBITS;      /* number of bits resulting bytes can hold */
-    padding = m - aa->nbits;  /* number of pad bits */
+    n = (aa->nbits + LEN_PAD_BITS + 6) / 7;  /* number of resulting bytes */
+    m = 7 * n - LEN_PAD_BITS;    /* number of bits resulting bytes can hold */
+    padding = m - aa->nbits;     /* number of pad bits */
     assert(0 <= padding && padding < 7);
 
     result = PyBytes_FromStringAndSize(NULL, n);
@@ -900,9 +883,8 @@ chdi_new(PyObject *module, PyObject *args)
     Py_ssize_t count_sum;
     chdi_obj *it;       /* iterator object to be returned */
 
-    if (!PyArg_ParseTuple(args, "OOO:count_n", &a, &count, &symbol))
-        return NULL;
-    if (ensure_bitarray(a) < 0)
+    if (!PyArg_ParseTuple(args, "O!OO:count_n",
+                          bitarray_type_obj, &a, &count, &symbol))
         return NULL;
     if (!PySequence_Check(count))
         return PyErr_Format(PyExc_TypeError, "count expected to be sequence, "
@@ -1041,15 +1023,6 @@ static PyTypeObject CHDI_Type = {
 
 /* --------------------------------------------------------------------- */
 
-/* Set bitarray_type_obj (bato).  This function must be called before any
-   other Python function in this module. */
-static PyObject *
-set_bato(PyObject *module, PyObject *obj)
-{
-    bitarray_type_obj = obj;
-    Py_RETURN_NONE;
-}
-
 static PyMethodDef module_functions[] = {
     {"count_n",   (PyCFunction) count_n,   METH_VARARGS, count_n_doc},
     {"rindex",    (PyCFunction) r_index,   METH_VARARGS, rindex_doc},
@@ -1067,7 +1040,6 @@ static PyMethodDef module_functions[] = {
     {"_vl_decode",(PyCFunction) vl_decode, METH_VARARGS, 0},
     {"canonical_decode",
                   (PyCFunction) chdi_new,  METH_VARARGS, chdi_doc},
-    {"_set_bato", (PyCFunction) set_bato,  METH_O,       0},
     {NULL,        NULL}  /* sentinel */
 };
 
@@ -1086,7 +1058,14 @@ PyInit__util(void)
 init_util(void)
 #endif
 {
-    PyObject *m;
+    PyObject *m, *bitarray_module;
+
+    if ((bitarray_module = PyImport_ImportModule("bitarray")) == NULL)
+        goto error;
+    bitarray_type_obj = PyObject_GetAttrString(bitarray_module, "bitarray");
+    Py_DECREF(bitarray_module);
+    if (bitarray_type_obj == NULL)
+        goto error;
 
 #ifdef IS_PY3K
     m = PyModule_Create(&moduledef);
